const fs = require('fs');

const is = (templates, schema, variable, expectedType, validatorRef) => {
  if ((typeof expectedType) === 'string') {
    const params = expectedType.split(' ');
    const component = params[0].toLowerCase();
    const args = params.slice(1).join(' ') || '';
    if (templates[component] !== undefined) {
      if (templates[component].archetype === undefined && (typeof templates[component].archetype) !== 'string') {
        throw new ReferenceError(`Template '${component}' does not export required interface property: archetype.`);
      }
      if (templates[component].archetype !== 'any' && (typeof variable) !== templates[component].archetype) {
        return false;
      }
      if (templates[component].performCheck !== undefined && (typeof templates[component].performCheck) === 'function') {
        return templates[component].performCheck(variable, schema, args, validatorRef);
      }

      throw new ReferenceError(`Template '${component}' does not implement required interface function: performCheck`);
    } else {
      throw new ReferenceError(`Unknown template type '${component}' provided to function: check.is`);
    }
  } else if ((typeof expectedType) === 'object' && !Array.isArray(expectedType)) {
    if (templates.scheme !== undefined) {
      if (templates.scheme.archetype === undefined && (typeof templates.scheme.archetype) !== 'string') {
        throw new ReferenceError('Template \'scheme\' does not export required interface property: archetype.');
      }
      if (templates.scheme.archetype !== 'any' && (typeof variable) !== templates.scheme.archetype) {
        return false;
      }
      if (templates.scheme.performCheck !== undefined && (typeof templates.scheme.performCheck) === 'function') {
        return templates.scheme.performCheck(variable, schema, expectedType, validatorRef);
      }

      throw new ReferenceError('Template \'scheme\' does not implement required interface function: performCheck');
    } else {
      throw new ReferenceError('Unknown template type \'scheme\' provided to function: check.is');
    }
  } else {
    throw new TypeError(`Input to check.is matcher must be a string or object (got ${typeof expectedType}).`);
  }
};

const has = (variable, path) => {
  const search = (obj, searchPath, currentPath) => {
    let found = false;
    let current = currentPath;

    if (searchPath === currentPath) {
      found = true;
    }

    if (!found) {
      Object.keys(obj).forEach((property) => {
        if ((typeof obj[property]) === 'object') {
          if (current !== '') {
            current += '.';
          }
          current += property;
          if (search(obj[property], searchPath, current)) {
            found = true;
          }
        } else if (searchPath === `${current}.${property}`) {
          found = true;
        }
      });
    }

    return found;
  };

  return search(variable, path, '');
};

const getFunctions = (templates, schema, variable, validatorRef) => {
  const functions = {
    is: (type) => is(templates, schema, variable, type, validatorRef),
  };

  if ((typeof variable) === 'object' && !Array.isArray(variable)) {
    functions.has = (propPath) => has(variable, propPath);
  }

  return functions;
};

const createScheme = (schema, schemeName, obj) => {
  if ((typeof schemeName) !== 'string') throw new TypeError('Scheme name must be a string.');
  if ((typeof obj) !== 'object') throw new TypeError('Scheme must be an object.');
  if (Array.isArray(obj)) throw new TypeError('Scheme must be a strict object (arrays not allowed).');

  const sch = schema;
  sch[schemeName] = obj;

  return sch;
};

const schema = (preloadedSchema) => {
  return preloadedSchema;
};

const loadTemplates = () => {
  const obj = {};

  fs.readdirSync('./lib/components').forEach((file) => {
    const load = (new RegExp(/^[A-Za-z0-9-]+.component.js$/gi)).test(file);
    if (load) {
      obj[file.split('.')[0].toLowerCase()] = require(`./components/${file}`);
    }
  });

  return obj;
};

const validator = (opt) => {
  let preloadedSchema = {};
  const validatorOptions = opt; // eslint-disable-line no-unused-vars
  const preloadedTemplates = loadTemplates();

  return {
    check: (variable) => {
      const validatorRef = validator(opt);
      Object.keys(preloadedSchema).forEach((schemeName) => {
        validatorRef.loadScheme(schemeName, preloadedSchema[schemeName]);
      });
      return getFunctions(preloadedTemplates, preloadedSchema, variable, validatorRef);
    },
    loadScheme: (schemeName, obj) => {
      try {
        preloadedSchema = createScheme(schema, schemeName, obj);
        return true;
      } catch (e) {
        return false;
      }
    },
    templates: () => {
      return preloadedTemplates;
    },
    schema: () => {
      return preloadedSchema;
    },
  };
};

module.exports = validator;
